# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####


bl_info = {
    "name": "O Beam",
    "author": "Laurent Tesson",
    "version": (1, 0),
    "blender": (2, 80, 0),
    "location": "View3D > Add > Mesh > O Beam",
    "description": "Adds a O Beam Profil",
    "warning": "",
    "wiki_url": "",
    "category": "Add Mesh",
}

import bpy
import mathutils
import os
from math import *
import math
from bpy.types import Operator
from bpy.props import FloatVectorProperty
from bpy_extras.object_utils import AddObjectHelper, object_data_add
from mathutils import Vector
from bpy.props import (
        BoolProperty,
        EnumProperty,
        FloatProperty,
        IntProperty,
        )
        
def add_obeamobject(self, context, prefs):
    scaleBeam = (prefs.scaleMesh)
    xBeam = ((self.beamX)/scaleBeam)
    zBeam = ((self.beamZ)/scaleBeam)
    eZ = 0
    nbZ = 1
    wBeam = ((self.beamW)/scaleBeam)
    resolA = self.resolutionA
    
    if zBeam > 0:
        nbZ = 2
        
    verts = []
    
    for i in range(0, nbZ):
        #arc
        i = 1
        for i in range(0, resolA):
            x_float = ((sin(math.radians((360/resolA)*i)))*xBeam)+(xBeam)
            y_float = ((cos(math.radians((360/resolA)*i)))*xBeam)+(xBeam)
            verts.append((x_float, y_float, eZ))
            x_float = ((sin(math.radians((360/resolA)*i)))*(xBeam-wBeam))+(xBeam)
            y_float = ((cos(math.radians((360/resolA)*i)))*(xBeam-wBeam))+(xBeam)
            verts.append((x_float, y_float, eZ))
        eZ = zBeam
    
    edges = []
    
    n_verts = len(verts)
    nbFace = (int)(n_verts/2)-1
    nbforE = (int)(n_verts/2)-1
    
    faces = []    
    
    if zBeam == 0:
        j = 0
        for i in range(0, nbFace):
            faces.append((j, j+1, j+3, j+2))
            j = j + 2
        faces.append((n_verts-2, n_verts-1, 1, 0))
    else:
        nbFace = (int)(n_verts/4)-1
        j = 0
        for i in range(0, nbFace):
            faces.append((j, j+1, j+3, j+2))
            j = j + 2
        faces.append((nbforE-1, nbforE, 1, 0))
        j = nbforE + 1
        for i in range(0, nbFace):
            faces.append((j, j+1, j+3, j+2))
            j = j + 2 
        faces.append((n_verts-2, n_verts-1, nbforE+2, nbforE+1))
        j = 0
        for i in range(0, nbFace):
            faces.append((j, j+nbforE+1, j+nbforE+3, j+2))
            faces.append((j+1, j+nbforE+2, j+nbforE+4, j+3))
            j = j + 2
        faces.append((nbforE-1, 0, nbforE+1, n_verts-2))
        faces.append((nbforE, 1, nbforE+2, n_verts-1))
        
    
    mesh = bpy.data.meshes.new(name="O_Beam")
    mesh.from_pydata(verts, edges, faces)
    object_data_add(context, mesh, operator=self)
    
    if self.rDouble:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.remove_doubles()
        bpy.ops.object.mode_set(mode='OBJECT')

    if zBeam == 0:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.normals_make_consistent(inside=True)
        bpy.ops.object.mode_set(mode='OBJECT')
    else:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.normals_make_consistent(inside=False)
        bpy.ops.object.mode_set(mode='OBJECT')
    

class OBJECT_OT_add_obeamobject(Operator, AddObjectHelper):
    """Create a new O Beam"""
    bl_idname = "mesh.add_obeamobject"
    bl_label = "Add O Beam"
    bl_options = {'REGISTER', 'UNDO'}

    #scale: FloatProperty(
    #        name="Scale",
    #        min=0.01, max=1000,
    #        default=1
    #        )
    beamX: FloatProperty(
            name="Radius",
            min=0.001, max=1000,
            default=1
            )
    beamZ: FloatProperty(
            name="Length Z",
            min=0, max=10000,
            default=2
            )
    beamW: FloatProperty(
            name="Thickness",
            min=0.001, max=1000,
            default=0.04
            )
    resolutionA: IntProperty(
            name="Resolution",
            min=3, max=1000,
            default=32
            )
    rDouble: BoolProperty(
            name="Remove doubles vertices",
            default=True,
            description="Remove doubles vertices"
            )

    def execute(self, context):
        prefs = bpy.context.preferences.addons['add_mesh_industrial_objects'].preferences
        add_obeamobject(self, context, prefs)        

        return {'FINISHED'}


# Registration

def add_obeamobject_button(self, context, prefs):
    self.layout.operator(
        OBJECT_OT_add_obeamobject.bl_idname,
        text="Add O Beam",
        icon='PLUGIN')


def register():
    bpy.utils.register_class(OBJECT_OT_add_obeamobject)
    bpy.types.VIEW3D_MT_mesh_add.append(add_obeamobject_button)


def unregister():
    bpy.utils.unregister_class(OBJECT_OT_add_obeamobject)
    bpy.types.VIEW3D_MT_mesh_add.remove(add_obeamobject_button)


if __name__ == "__main__":
    register()