# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####


bl_info = {
    "name": "U Beam",
    "author": "Laurent Tesson",
    "version": (1, 0),
    "blender": (2, 80, 0),
    "location": "View3D > Add > Mesh > U Beam",
    "description": "Adds a U Beam Profil",
    "warning": "",
    "wiki_url": "",
    "category": "Add Mesh",
}

import bpy
import mathutils
import os
from math import *
import math
from bpy.types import Operator
from bpy.props import FloatVectorProperty
from bpy_extras.object_utils import AddObjectHelper, object_data_add
from mathutils import Vector
from bpy.props import (
        BoolProperty,
        EnumProperty,
        FloatProperty,
        IntProperty,
        )
        
def add_ubeamobject(self, context, prefs):
    scaleBeam = (prefs.scaleMesh)
    xBeam = ((self.beamX)/scaleBeam)
    yBeam = ((self.beamY)/scaleBeam)
    zBeam = ((self.beamZ)/scaleBeam)
    eZ = 0
    nbZ = 1
    wBeam = ((self.beamW)/scaleBeam)
    wxBeam = ((self.beamWx)/scaleBeam)
    tBeam = ((self.beamT)*10)
    rABeam = ((self.rA)/scaleBeam)
    rBBeam = ((self.rB)/scaleBeam)   
    rCBeam = ((self.rC)/scaleBeam)   
    resolA = self.resolutionA
    resolB = self.resolutionB
    resolC = self.resolutionC
    
    if zBeam > 0:
        nbZ = 2

    verts = []
    
    for i in range(0, nbZ):
        verts.append((0, rCBeam, eZ))
        verts.append((0, yBeam-rCBeam, eZ))
        #arc
        i = 1
        invI = resolC - 1
        for i in range(1, resolC):
            x_float = ((sin(math.radians((90/resolC)*invI)))*rCBeam*(-1))+(rCBeam)
            y_float = ((cos(math.radians((90/resolC)*invI)))*rCBeam)+(yBeam-rCBeam)
            verts.append((x_float, y_float, eZ))
            invI = invI - 1 
        verts.append((rCBeam, yBeam, eZ))
        verts.append((xBeam, yBeam, eZ))
        #arc
        i = 1
        invI = resolA - 1
        for i in range(1, resolA):
            x_float = ((sin(math.radians((90/resolA)*invI)))*rABeam)+(xBeam-rABeam)
            y_float = ((cos(math.radians((90/resolA)*invI)))*rABeam*(-1))+(yBeam-wBeam+rABeam+(((xBeam-rABeam-rBBeam-wxBeam)*tBeam)/1000)-((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000))
            verts.append((x_float, y_float, eZ))
            invI = invI - 1 
        if tBeam == 0:
            verts.append((wxBeam+rBBeam, yBeam-wBeam, eZ))
        #arc
        i = 1
        for i in range(1, resolB):
            x_float = ((sin(math.radians((90/resolB)*i)))*rBBeam*(-1))+(wxBeam+rBBeam)
            y_float = ((cos(math.radians((90/resolB)*i)))*rBBeam)+(yBeam-wBeam-rBBeam-((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000))
            verts.append((x_float, y_float, eZ))
        verts.append((wxBeam, (yBeam-wBeam-rBBeam-((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000)), eZ))
        verts.append((wxBeam, wBeam+rBBeam+((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000), eZ))
        #arc
        i = 1
        invI = resolB - 1
        for i in range(1, resolB):
            x_float = ((sin(math.radians((90/resolB)*invI)))*rBBeam*(-1))+(wxBeam+rBBeam)
            y_float = ((cos(math.radians((90/resolB)*invI)))*rBBeam*(-1))+(wBeam+rBBeam+((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000))
            verts.append((x_float, y_float, eZ))
            invI = invI - 1 
        if tBeam == 0:
            verts.append((wxBeam+rBBeam, wBeam, eZ)) 
        #arc
        i = 1
        for i in range(1, resolA):
            x_float = ((sin(math.radians((90/resolA)*i)))*rABeam)+(xBeam-rABeam)
            y_float = ((cos(math.radians((90/resolA)*i)))*rABeam)+(wBeam-rABeam-(((xBeam-rABeam-rBBeam-wxBeam)*tBeam)/1000)+((((xBeam/2)-rBBeam-wxBeam)*tBeam)/1000))
            verts.append((x_float, y_float, eZ))
            invI = invI - 1
        verts.append((xBeam, 0, eZ))
        verts.append((rCBeam, 0, eZ))
        #arc
        i = 1
        for i in range(1, resolC):
            x_float = ((sin(math.radians((90/resolC)*i)))*rCBeam*(-1))+(rCBeam)
            y_float = ((cos(math.radians((90/resolC)*i)))*rCBeam*(-1))+(rCBeam)
            verts.append((x_float, y_float, eZ))            
        eZ = zBeam 

    edges = []
    
    n_verts = len(verts)
    nbforE = (int)(n_verts/2)-1
    
    faces = []    
    if zBeam == 0:   
        faces.append(range(n_verts))
    else:
        faces.append(range(0, nbforE))
        faces.append(range(nbforE+1, n_verts))
        j = 0
        for i in range(0, nbforE):
            faces.append((j, j+nbforE+1, j+nbforE+2, j+1))
            j = j + 1
        faces.append((nbforE, n_verts-1, nbforE+1, 0))
    
    mesh = bpy.data.meshes.new(name="U_Beam")
    mesh.from_pydata(verts, edges, faces)
    object_data_add(context, mesh, operator=self)
    
    if self.rDouble:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.remove_doubles()
        bpy.ops.object.mode_set(mode='OBJECT')

    if zBeam == 0:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.normals_make_consistent(inside=True)
        bpy.ops.object.mode_set(mode='OBJECT')
    else:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.normals_make_consistent(inside=False)
        bpy.ops.object.mode_set(mode='OBJECT')
        

class OBJECT_OT_add_ubeamobject(Operator, AddObjectHelper):
    """Create a new U Beam"""
    bl_idname = "mesh.add_ubeamobject"
    bl_label = "Add U Beam"
    bl_options = {'REGISTER', 'UNDO'}

    #scale: FloatProperty(
    #        name="Scale",
    #        min=0.01, max=1000,
    #        default=1
    #        )
    beamX: FloatProperty(
            name="Length X",
            min=0.001, max=1000,
            default=1
            )
    beamY: FloatProperty(
            name="Length Y",
            min=0.001, max=1000,
            default=2
            )
    beamZ: FloatProperty(
            name="Length Z",
            min=0, max=10000,
            default=2
            )
    beamW: FloatProperty(
            name="Thickness 1",
            min=0.001, max=1000,
            default=0.17
            )
    beamWx: FloatProperty(
            name="Thickness 2",
            min=0.001, max=1000,
            default=0.12
            )
    beamT: FloatProperty(
            name="Taper",
            min=0, max=1000,
            default=14
            )
    rA: FloatProperty(
            name="Radius 1",
            min=0, max=1000,
            default=0.09
            )
    resolutionA: IntProperty(
            name="Resolution",
            min=1, max=1000,
            default=10
            )
    rB: FloatProperty(
            name="Radius 2",
            min=0, max=1000,
            default=0.17
            )
    resolutionB: IntProperty(
            name="Resolution",
            min=1, max=1000,
            default=10
            )
    rC: FloatProperty(
            name="Radius 3",
            min=0, max=1000,
            default=0
            )
    resolutionC: IntProperty(
            name="Resolution",
            min=1, max=1000,
            default=1
            )
    rDouble: BoolProperty(
            name="Remove doubles vertices",
            default=True,
            description="Remove doubles vertices"
            )

    def execute(self, context):        
        prefs = bpy.context.preferences.addons['add_mesh_industrial_objects'].preferences
        add_ubeamobject(self, context, prefs)

        return {'FINISHED'}


# Registration

def add_ubeamobject_button(self, context, prefs):
    self.layout.operator(
        OBJECT_OT_add_ubeamobject.bl_idname,
        text="Add U Beam",
        icon='PLUGIN')


def register():
    bpy.utils.register_class(OBJECT_OT_add_ubeamobject)
    bpy.types.VIEW3D_MT_mesh_add.append(add_ubeamobject_button)


def unregister():
    bpy.utils.unregister_class(OBJECT_OT_add_ubeamobject)
    bpy.types.VIEW3D_MT_mesh_add.remove(add_ubeamobject_button)


if __name__ == "__main__":
    register()